/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 * Please see the LICENSE file or "Copyright notice" in builder.cpp for details.
 */

/*
 * QtInfoBox.cpp
 *
 *  Created on: Mar 4, 2010
 *      Author: crueger
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Views/Qt4/QtInfoBox.hpp"

#include <iostream>
#include <QAbstractItemView>

#include "CodePatterns/MemDebug.hpp"

#include "molecule.hpp"
#include "Element/element.hpp"

using namespace std;

/***************** Basic structure for tab layout ***********/

QtInfoBox::QtInfoBox() :
    QTabWidget(),
    curAtom(NULL), nextAtom(NULL),
    page_mol(NULL), page_atom(NULL)
{
  timer = new QTimer(this);
  timer->setSingleShot(true);

  connect(timer, SIGNAL(timeout()), this, SLOT(timerTimeout()));
}

QtInfoBox::~QtInfoBox()
{
  clearTabs();
}

void QtInfoBox::atomHover(const atom *_atom)
{
  nextAtom = _atom;
  timer->start(500);
}

void QtInfoBox::timerTimeout()
{
  if (nextAtom)
    showAtom(nextAtom);
}

void QtInfoBox::clearTabs()
{
  if (page_atom){
    //removeTab(indexOf(page_atom));
    delete(page_atom);
    page_atom = NULL;
  }
  if (page_mol){
    //removeTab(indexOf(page_mol));
    delete(page_mol);
    page_mol = NULL;
  }
}

void QtInfoBox::showAtom(const atom *_atom)
{
  // Remove old tabs.
  clearTabs();

  curAtom = _atom;

  // Show new tabs.
  if (curAtom){
    page_atom = new QtAtomInfoPage(curAtom);
    addTab(page_atom, "Atom");
    connect(page_atom, SIGNAL(atomKilled()), this, SLOT(clearTabs()));

    if (curAtom->getMolecule()){
      page_mol = new QtMoleculeInfoPage(curAtom->getMolecule());
      addTab(page_mol, "Molecule");
      connect(page_mol, SIGNAL(moleculeKilled()), this, SLOT(clearTabs()));
    }
  }
}

/************************ Tab for single Atoms ********************/

static void addInfo(QTreeWidget *info, const QString &key, const QString &value)
{
  QTreeWidgetItem *treeItem = new QTreeWidgetItem(info);
  treeItem->setText(0, key);
  treeItem->setText(1, value);
}

QtAtomInfoPage::QtAtomInfoPage(const atom *_atom) :
  Observer("QTAtomPage"),
  atomRef(_atom)
{
  atomRef->signOn(this);

  info = new QTreeWidget(this);
  info->setColumnCount(2);
  QStringList header;
  header << "data";
  header << "value";
  info->setHeaderLabels(header);

  addInfo(info, "Element", QString(atomRef->getElement().getName().c_str()));
  addInfo(info, "Mass", QString("%1").arg(atomRef->getMass()));
  addInfo(info, "Charge", QString("%1").arg(atomRef->getCharge()));
  addInfo(info, "Position", QString(toString(atomRef->getPosition()).c_str()));
  addInfo(info, "Bonds", QString("%1").arg(atomRef->getListOfBonds().size()));
}

QtAtomInfoPage::~QtAtomInfoPage()
{
  if (atomRef)
    atomRef->signOff(this);
}

void QtAtomInfoPage::update(Observable *subject){
  /*if(name != atomRef->name){
    name = atomRef->name;
    emit nameChanged(this,name);
  }*/
}

void QtAtomInfoPage::subjectKilled(Observable *subject){
  atomRef = NULL;
  emit atomKilled();
}

/************************ Tab for single Molecules *****************/

QtMoleculeInfoPage::QtMoleculeInfoPage(const molecule *_mol) :
    Observer("QTMoleculePage"),
    mol(_mol)
{
  mol->signOn(this);

  info = new QTreeWidget(this);
  info->setColumnCount(2);
  QStringList header;
  header << "data";
  header << "value";
  info->setHeaderLabels(header);

  addInfo(info, "Name", QString(mol->getName().c_str()));
  addInfo(info, "Formula", QString(mol->getFormula().toString().c_str()));
  addInfo(info, "Atoms", QString("%1").arg(mol->getAtomCount()));
  addInfo(info, "Bonds", QString("%1").arg(mol->getBondCount()));
}

QtMoleculeInfoPage::~QtMoleculeInfoPage(){
  if (mol)
    mol->signOff(this);
}

void QtMoleculeInfoPage::update(Observable *subject){  std::cout << "tab mol update\n";
}

void QtMoleculeInfoPage::subjectKilled(Observable *subject){
  mol = NULL;
  emit moleculeKilled();
}
